Package: edgarfundamentals
Title: Retrieve Fundamental Financial Data from SEC 'EDGAR'
Version: 0.1.2
Authors@R: person(
    given   = "Robert P",
    family  = "Schumaker",
    role    = c("aut", "cre"),
    email   = "rob.schumaker@gmail.com",
    comment = c(ORCID = "0000-0003-2977-1856"))
Description: Provides a simple, ticker-based interface for retrieving fundamental
    financial data from the United States Securities and Exchange Commission's
    'EDGAR' 'XBRL' API <https://www.sec.gov/edgar/sec-api-documentation>. Functions
    return key financial ratios including earnings per share, return on equity,
    return on assets, debt-to-equity, current ratio, gross margin, operating
    margin, net margin, price-to-earnings, price-to-book, and dividend yield
    for any publicly traded U.S. company. Data is sourced directly from company
    10-K annual filings, requiring no API key or paid subscription. Designed for
    use in quantitative finance courses and research workflows.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.1.0)
Imports: cli, dplyr (>= 1.1.0), httr (>= 1.4.0), jsonlite (>= 1.8.0),
        rlang, tidyquant (>= 1.0.0)
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
URL: https://github.com/robschumaker/edgarfundamentals
BugReports: https://github.com/robschumaker/edgarfundamentals/issues
NeedsCompilation: no
Packaged: 2026-03-26 17:37:17 UTC; rschumaker
Author: Robert P Schumaker [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2977-1856>)
Maintainer: Robert P Schumaker <rob.schumaker@gmail.com>
Repository: CRAN
Date/Publication: 2026-03-27 20:10:02 UTC
Built: R 4.6.0; ; 2026-04-25 13:40:38 UTC; unix

# Generated by roxygen2: do not edit by hand

export(get_cik)
export(get_filing_history)
export(get_fundamentals)
export(get_fundamentals_batch)
importFrom(dplyr,arrange)
importFrom(dplyr,desc)
importFrom(dplyr,filter)
importFrom(dplyr,pull)
importFrom(dplyr,slice)
importFrom(httr,GET)
importFrom(httr,add_headers)
importFrom(httr,content)
importFrom(jsonlite,fromJSON)
importFrom(rlang,.data)
importFrom(tidyquant,tq_get)

# edgarfundamentals

**Retrieve Fundamental Financial Data from SEC EDGAR**

`edgarfundamentals` provides a simple, ticker-based interface for retrieving
key fundamental financial ratios directly from SEC EDGAR 10-K filings. No API
key or paid subscription is required.

## Installation

```r
# From CRAN (once published)
install.packages("edgarfundamentals")

# Development version
devtools::install_github("robschumaker/edgarfundamentals")
```

## Setup

The SEC requests that automated tools identify themselves via a User-Agent
header. Set this once per session:

```r
options(edgarfundamentals.user_agent = "Your Name your@email.com")
```

## Functions

| Function | Description |
|----------|-------------|
| `get_cik(symbol)` | Translate a ticker to its SEC Central Index Key |
| `get_fundamentals(symbol, to_date)` | Key ratios for one stock from its most recent 10-K |
| `get_fundamentals_batch(symbols, to_date)` | Key ratios for a portfolio of stocks |
| `get_filing_history(symbol, form_type, n)` | Recent SEC filing history for a stock |

## Quick Example

```r
library(edgarfundamentals)
options(edgarfundamentals.user_agent = "Jane Smith jane@example.com")

# Single stock
get_fundamentals("LLY", to_date = "2024-12-31")

# Portfolio
healthcare <- c("UNH", "PFE", "MRK", "ABT", "LLY", "CVS", "AMGN")
get_fundamentals_batch(healthcare, to_date = "2024-12-31")
```

## Ratios Returned

`get_fundamentals()` and `get_fundamentals_batch()` return the following:

| Ratio | Description | Source |
|-------|-------------|--------|
| `EPS` | Diluted Earnings Per Share (USD) | EDGAR |
| `NetIncome` | Net Income (USD) | EDGAR |
| `Revenue` | Total Revenue (USD) | EDGAR |
| `ROE` | Return on Equity (%) | EDGAR |
| `ROA` | Return on Assets (%) | EDGAR |
| `DE` | Debt-to-Equity ratio | EDGAR |
| `CurrentRatio` | Current Assets / Current Liabilities | EDGAR |
| `GrossMargin` | Gross Profit as % of Revenue | EDGAR |
| `OperatingMargin` | Operating Income as % of Revenue | EDGAR |
| `NetMargin` | Net Income as % of Revenue | EDGAR |
| `PE` | Price-to-Earnings ratio | EDGAR + Yahoo |
| `PB` | Price-to-Book ratio | EDGAR + Yahoo |
| `DIV` | Dividend Yield (%) | EDGAR + Yahoo |



After cloning, generate documentation and check the package before submission:

```r
devtools::document()   # generates man/ files from roxygen2 comments
devtools::check()      # runs R CMD check -- must pass with 0 errors, 0 warnings
devtools::build()      # builds the .tar.gz for CRAN submission
```

The `man/` directory is generated automatically by `devtools::document()` and
is not tracked in version control.

## Data Source

All financial statement data comes from the SEC EDGAR XBRL API
(`data.sec.gov`), specifically the `companyfacts` endpoint. Ratios reflect
the most recently completed fiscal year from 10-K filings. PE, PB, and DIV
additionally use current market prices from Yahoo Finance via `tidyquant`.

## License

MIT © Robert P Schumaker

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  eval     = FALSE
)

## ----user-agent---------------------------------------------------------------
# library(edgarfundamentals)
# 
# options(edgarfundamentals.user_agent = "Jane Smith jane@example.com")

## ----get-cik------------------------------------------------------------------
# get_cik("LLY")   # Eli Lilly
# get_cik("LMT")   # Lockheed Martin

## ----single-------------------------------------------------------------------
# get_fundamentals("LLY", to_date = "2024-12-31")

## ----batch--------------------------------------------------------------------
# healthcare <- c("UNH", "PFE", "MRK", "ABT", "LLY", "CVS", "AMGN")
# defense    <- c("LMT", "RTX", "NOC", "GD",  "HII", "LHX", "LDOS")
# 
# healthcare.fund <- get_fundamentals_batch(healthcare, to_date = "2024-12-31")
# defense.fund    <- get_fundamentals_batch(defense,    to_date = "2024-12-31")
# 
# healthcare.fund
# defense.fund

## ----screening----------------------------------------------------------------
# library(dplyr)
# 
# # Growth screen: high EPS stocks relative to peers signal strong earnings
# healthcare.fund |> arrange(desc(EPS))
# 
# # Value screen: low PE and low PB suggest the market is pricing the stock cheaply
# # High dividend yield is also characteristic of value stocks
# defense.fund |> filter(PE < 20 & PB < 3) |> select(symbol, PE, PB, DIV, ROE)
# 
# # GARP screen (Growth at a Reasonable Price): PE below EPS
# # Stocks where the market is not overcharging for growth
# healthcare.fund |> filter(EPS > 0 & PE < EPS) |> select(symbol, PE, EPS)
# 
# # Profitability screen: strong margins signal pricing power and operational efficiency
# bind_rows(healthcare.fund, defense.fund) |>
#   filter(GrossMargin > 40 & OperatingMargin > 15) |>
#   arrange(desc(NetMargin)) |>
#   select(symbol, GrossMargin, OperatingMargin, NetMargin, ROA)
# 
# # Liquidity screen: current ratio above 1.5 and low leverage
# bind_rows(healthcare.fund, defense.fund) |>
#   filter(CurrentRatio > 1.5 & DE < 1) |>
#   arrange(desc(CurrentRatio)) |>
#   select(symbol, CurrentRatio, DE, ROE)

## ----filing-history-----------------------------------------------------------
# # Five most recent annual reports for Lockheed Martin
# get_filing_history("LMT", form_type = "10-K", n = 5)
# 
# # Four most recent quarterly reports for Eli Lilly
# get_filing_history("LLY", form_type = "10-Q", n = 4)

#' Retrieve fundamental financial ratios for a portfolio of stocks
#'
#' A vectorized wrapper around \code{\link{get_fundamentals}} that accepts a
#' vector of ticker symbols and returns a tidy data frame with one row per
#' stock. Failed lookups are recorded as \code{NA} rather than stopping
#' execution, so a single problematic ticker does not interrupt the batch.
#'
#' @param symbols A character vector of stock ticker symbols
#'   (e.g. \code{c("LLY", "PFE", "UNH")}). Case-insensitive.
#' @param to_date A character string in \code{"YYYY-MM-DD"} format. Passed
#'   to \code{\link{get_fundamentals}} for each symbol. Defaults to today's
#'   date.
#'
#' @return A data frame with one row per symbol and the following columns:
#'   \code{symbol}, \code{CIK}, \code{EPS}, \code{NetIncome}, \code{Revenue},
#'   \code{ROE}, \code{ROA}, \code{DE}, \code{CurrentRatio},
#'   \code{GrossMargin}, \code{OperatingMargin}, \code{NetMargin},
#'   \code{PE}, \code{PB}, \code{DIV}. See \code{\link{get_fundamentals}}
#'   for definitions. Rows where data retrieval failed contain \code{NA}
#'   for all ratio columns.
#'
#' @details
#' Each symbol requires two SEC EDGAR API calls (one for the CIK lookup and
#' one for the companyfacts data) plus one Yahoo Finance call for the current
#' price. A 0.5-second pause is inserted after each companyfacts call to
#' respect the SEC rate limit of 10 requests per second. For a portfolio of
#' 14 stocks, expect a total retrieval time of approximately 20--30 seconds.
#'
#' Set your User-Agent once per session:
#' \code{options(edgarfundamentals.user_agent = "Your Name your@@email.com")}
#'
#' @examples
#' \dontrun{
#' options(edgarfundamentals.user_agent = "Jane Smith jane@@example.com")
#'
#' healthcare <- c("UNH", "PFE", "MRK", "ABT", "LLY", "CVS", "AMGN")
#' get_fundamentals_batch(healthcare, "2024-12-31")
#'
#' defense <- c("LMT", "RTX", "NOC", "GD", "HII", "LHX", "LDOS")
#' get_fundamentals_batch(defense, "2024-12-31")
#' }
#'
#' @seealso \code{\link{get_fundamentals}} for single-stock retrieval.
#' @export
get_fundamentals_batch <- function(symbols, to_date = as.character(Sys.Date())) {

  na_row <- c(CIK             = NA_real_, EPS             = NA_real_,
              NetIncome       = NA_real_, Revenue         = NA_real_,
              ROE             = NA_real_, ROA             = NA_real_,
              DE              = NA_real_, CurrentRatio    = NA_real_,
              GrossMargin     = NA_real_, OperatingMargin = NA_real_,
              NetMargin       = NA_real_, PE              = NA_real_,
              PB              = NA_real_, DIV             = NA_real_)

  results <- lapply(symbols, function(sym) {
    tryCatch(
      get_fundamentals(sym, to_date),
      error = function(e) {
        message("edgarfundamentals: failed for ", sym, " -- ", conditionMessage(e))
        na_row
      }
    )
  })

  out <- as.data.frame(do.call(rbind, results))
  out <- cbind(symbol = symbols, out, row.names = NULL)
  out
}

#' @importFrom rlang .data
NULL
# Internal helpers -- not exported

# Retrieve and validate the User-Agent string from options.
# SEC policy requires requests to identify the requester.
# Users should set this once per session:
#   options(edgarfundamentals.user_agent = "Your Name your@email.com")
edgar_ua <- function() {
  ua <- getOption("edgarfundamentals.user_agent",
                  default = "edgarfundamentals R package https://cran.r-project.org/package=edgarfundamentals")
  httr::add_headers("User-Agent" = ua)
}

# Zero-pad a CIK to 10 digits as required by EDGAR URL format.
pad_cik <- function(cik) {
  formatC(as.integer(cik), width = 10, flag = "0")
}

# Extract the most recent 10-K value for a single XBRL concept.
# Tries each tag in order and returns the first non-NA result.
# primary_tag  -- the preferred us-gaap tag name (e.g. "EarningsPerShareDiluted")
# fallback_tags -- character vector of alternative tag names to try if primary fails
# unit          -- XBRL unit string (e.g. "USD/shares" or "USD")
# to_date       -- only consider filings with period end on or before this date
latest_10k <- function(facts, primary_tag, fallback_tags = NULL, unit, to_date) {
  for (tag in c(primary_tag, fallback_tags)) {
    result <- tryCatch({
      df <- as.data.frame(facts$facts$`us-gaap`[[tag]]$units[[unit]])
      if (nrow(df) == 0) next
      val <- df |>
        dplyr::filter(.data$form == "10-K", as.Date(.data$end) <= as.Date(to_date)) |>
        dplyr::arrange(dplyr::desc(.data$end), dplyr::desc(.data$filed)) |>
        dplyr::slice(1) |>
        dplyr::pull(.data$val)
      if (length(val) > 0 && !is.na(val)) return(as.numeric(val))
      NA_real_
    }, error = function(e) NA_real_)
    if (!is.na(result)) return(result)
  }
  NA_real_
}

#' Retrieve recent SEC filing history for a company
#'
#' Returns a data frame listing the most recent EDGAR filings of a specified
#' form type for a given company. Useful for verifying that a company has
#' filed the expected number of 10-K or 10-Q reports, or for retrieving
#' accession numbers needed to access specific filings.
#'
#' @param symbol A character string containing the stock ticker symbol
#'   (e.g. \code{"AAPL"}). Case-insensitive.
#' @param form_type A character string specifying the SEC form type to
#'   retrieve. Common values are \code{"10-K"} (annual report),
#'   \code{"10-Q"} (quarterly report), and \code{"8-K"} (current report).
#'   Defaults to \code{"10-K"}.
#' @param n An integer specifying the maximum number of filings to return.
#'   Defaults to \code{5}.
#'
#' @return A data frame with one row per filing and the following columns:
#' \describe{
#'   \item{symbol}{The ticker symbol passed to the function.}
#'   \item{accession_number}{The SEC accession number uniquely identifying
#'     the filing (e.g. \code{"0000320193-24-000123"}).}
#'   \item{filing_date}{The date the filing was submitted to EDGAR.}
#'   \item{report_date}{The period-end date covered by the filing.}
#'   \item{form}{The form type as recorded in EDGAR.}
#'   \item{primary_document}{The filename of the primary HTML document within
#'     the filing.}
#' }
#'
#' @details
#' Data is retrieved from the SEC EDGAR submissions API
#' (\url{https://data.sec.gov/submissions/}). The API returns up to the 1,000
#' most recent filings across all form types; older filings may not appear.
#'
#' Set your User-Agent once per session:
#' \code{options(edgarfundamentals.user_agent = "Your Name your@@email.com")}
#'
#' @examples
#' \dontrun{
#' options(edgarfundamentals.user_agent = "Jane Smith jane@@example.com")
#'
#' # Five most recent annual reports for Lockheed Martin
#' get_filing_history("LMT", form_type = "10-K", n = 5)
#'
#' # Most recent quarterly reports for Eli Lilly
#' get_filing_history("LLY", form_type = "10-Q", n = 4)
#' }
#'
#' @importFrom httr GET add_headers content
#' @importFrom jsonlite fromJSON
#' @importFrom dplyr filter
#' @export
get_filing_history <- function(symbol, form_type = "10-K", n = 5) {

  cik     <- get_cik(symbol)
  cik.pad <- pad_cik(cik)

  response <- httr::GET(
    paste0("https://data.sec.gov/submissions/CIK", cik.pad, ".json"),
    edgar_ua()
  )

  submissions <- httr::content(response, as = "text", encoding = "UTF-8") |>
    jsonlite::fromJSON()

  Sys.sleep(0.5)

  recent <- submissions$filings$recent

  all.filings <- data.frame(
    accession_number = recent$accessionNumber,
    filing_date      = recent$filingDate,
    report_date      = recent$reportDate,
    form             = recent$form,
    primary_document = recent$primaryDocument,
    stringsAsFactors = FALSE
  )

  filtered <- all.filings |>
    dplyr::filter(.data$form == form_type)

  if (nrow(filtered) == 0) {
    message("No '", form_type, "' filings found for ", symbol,
            " in the most recent EDGAR submissions.")
    return(data.frame())
  }

  out <- utils::head(filtered, n)
  cbind(symbol = symbol, out, row.names = NULL)
}

#' Retrieve key fundamental financial ratios for a single stock
#'
#' Pulls fundamental financial data directly from a company's most recent
#' annual 10-K filing in SEC EDGAR and computes key financial ratios. No API
#' key or paid subscription is required.
#'
#' @param symbol A character string containing the stock ticker symbol
#'   (e.g. \code{"LLY"}). Case-insensitive.
#' @param to_date A character string in \code{"YYYY-MM-DD"} format. The
#'   function returns ratios from the most recent 10-K with a period end date
#'   on or before this date. Defaults to today's date.
#'
#' @return A named numeric vector with the following elements:
#' \describe{
#'   \item{CIK}{SEC Central Index Key -- the company's unique EDGAR identifier.}
#'   \item{EPS}{Diluted Earnings Per Share (USD per share) from the most recent
#'     qualifying 10-K. Falls back to basic EPS if diluted is unavailable.}
#'   \item{NetIncome}{Net income attributable to the company (USD).}
#'   \item{Revenue}{Total revenue (USD).}
#'   \item{ROE}{Return on Equity as a percentage (NetIncome / StockholdersEquity
#'     * 100). A measure of how efficiently the company generates profit from
#'     shareholder capital.}
#'   \item{ROA}{Return on Assets as a percentage (NetIncome / TotalAssets
#'     * 100). A measure of how efficiently the company uses its assets.}
#'   \item{DE}{Debt-to-Equity ratio (LongTermDebt / StockholdersEquity). A
#'     measure of financial leverage.}
#'   \item{CurrentRatio}{Current Assets divided by Current Liabilities. A
#'     measure of short-term liquidity; values above 1 indicate the company
#'     can cover near-term obligations.}
#'   \item{GrossMargin}{Gross Profit as a percentage of Revenue. Measures
#'     pricing power and production efficiency.}
#'   \item{OperatingMargin}{Operating Income as a percentage of Revenue.
#'     Measures operational efficiency before interest and taxes.}
#'   \item{NetMargin}{Net Income as a percentage of Revenue. Bottom-line
#'     profitability after all expenses.}
#'   \item{PE}{Price-to-Earnings ratio, computed by dividing the most recent
#'     adjusted closing price (from Yahoo Finance via tidyquant) by EPS.
#'     Returns \code{NA} if EPS is zero or negative.}
#'   \item{PB}{Price-to-Book ratio, computed by dividing the most recent
#'     adjusted closing price by Book Value per Share (StockholdersEquity /
#'     SharesOutstanding). Returns \code{NA} if shares outstanding is zero
#'     or unavailable.}
#'   \item{DIV}{Dividend Yield as a percentage (DividendsPerShareDeclared /
#'     Price * 100). Returns \code{NA} if no dividend data is available.}
#' }
#'
#' @details
#' Financial statement values are extracted from XBRL-tagged 10-K filings via
#' the SEC EDGAR companyfacts API
#' (\url{https://data.sec.gov/api/xbrl/companyfacts/}). Because data comes
#' from annual filings, ratios reflect the most recently completed fiscal year
#' ending on or before \code{to_date}, not real-time values.
#'
#' Fallback XBRL tags are attempted automatically when a company uses a
#' non-standard tag name for a concept. A courtesy pause of 0.5 seconds is
#' inserted after the companyfacts API call to respect the SEC's rate limit
#' of 10 requests per second.
#'
#' Set your User-Agent once per session:
#' \code{options(edgarfundamentals.user_agent = "Your Name your@@email.com")}
#'
#' @examples
#' \dontrun{
#' options(edgarfundamentals.user_agent = "Jane Smith jane@@example.com")
#'
#' # Fundamentals for Eli Lilly as of end of 2024
#' get_fundamentals("LLY", "2024-12-31")
#'
#' # Fundamentals as of today
#' get_fundamentals("JNJ")
#' }
#'
#' @importFrom httr GET add_headers content
#' @importFrom jsonlite fromJSON
#' @importFrom dplyr filter arrange desc slice pull
#' @importFrom tidyquant tq_get
#' @export
get_fundamentals <- function(symbol, to_date = as.character(Sys.Date())) {

  cik     <- get_cik(symbol)
  cik.pad <- pad_cik(cik)

  facts <- httr::GET(
    paste0("https://data.sec.gov/api/xbrl/companyfacts/CIK", cik.pad, ".json"),
    edgar_ua()
  ) |>
    httr::content(as = "text", encoding = "UTF-8") |>
    jsonlite::fromJSON()

  Sys.sleep(0.5)

  # Extract raw values -- each concept tries a primary tag then fallback tags
  eps        <- latest_10k(facts, "EarningsPerShareDiluted",
                           fallback_tags = "EarningsPerShareBasic",
                           unit = "USD/shares", to_date = to_date)

  net.income <- latest_10k(facts, "NetIncomeLoss",
                           fallback_tags = c("ProfitLoss", "NetIncome"),
                           unit = "USD", to_date = to_date)

  revenue    <- latest_10k(facts, "RevenueFromContractWithCustomerExcludingAssessedTax",
                           fallback_tags = c("Revenues", "SalesRevenueNet",
                                             "SalesRevenueGoodsNet"),
                           unit = "USD", to_date = to_date)

  equity     <- latest_10k(facts, "StockholdersEquity",
                           fallback_tags = "StockholdersEquityIncludingPortionAttributableToNoncontrollingInterest",
                           unit = "USD", to_date = to_date)

  debt       <- latest_10k(facts, "LongTermDebt",
                           fallback_tags = c("LongTermDebtNoncurrent", "LongTermNotesPayable"),
                           unit = "USD", to_date = to_date)

  assets     <- latest_10k(facts, "Assets",
                           unit = "USD", to_date = to_date)

  current.assets <- latest_10k(facts, "AssetsCurrent",
                                unit = "USD", to_date = to_date)

  current.liab   <- latest_10k(facts, "LiabilitiesCurrent",
                                unit = "USD", to_date = to_date)

  gross.profit   <- latest_10k(facts, "GrossProfit",
                                unit = "USD", to_date = to_date)

  operating.inc  <- latest_10k(facts, "OperatingIncomeLoss",
                                unit = "USD", to_date = to_date)

  shares.out     <- latest_10k(facts, "CommonStockSharesOutstanding",
                                fallback_tags = c("CommonStockSharesIssuedAndOutstanding",
                                                  "EntityCommonStockSharesOutstanding"),
                                unit = "shares", to_date = to_date)

  div.per.share  <- latest_10k(facts, "CommonStockDividendsPerShareDeclared",
                                fallback_tags = c("CommonStockDividendsPerShareCashPaid",
                                                  "DividendsCommonStockCash"),
                                unit = "USD/shares", to_date = to_date)

  # Compute derived ratios
  roe <- if (!is.na(net.income) && !is.na(equity) && equity != 0)
    round(net.income / equity * 100, 2) else NA_real_

  roa <- if (!is.na(net.income) && !is.na(assets) && assets != 0)
    round(net.income / assets * 100, 2) else NA_real_

  de  <- if (!is.na(debt) && !is.na(equity) && equity != 0)
    round(debt / equity, 2) else NA_real_

  current.ratio <- if (!is.na(current.assets) && !is.na(current.liab) && current.liab != 0)
    round(current.assets / current.liab, 2) else NA_real_

  gross.margin <- if (!is.na(gross.profit) && !is.na(revenue) && revenue != 0)
    round(gross.profit / revenue * 100, 2) else NA_real_

  operating.margin <- if (!is.na(operating.inc) && !is.na(revenue) && revenue != 0)
    round(operating.inc / revenue * 100, 2) else NA_real_

  net.margin <- if (!is.na(net.income) && !is.na(revenue) && revenue != 0)
    round(net.income / revenue * 100, 2) else NA_real_

  # PE, PB, and DIV all use the most recent adjusted price from tidyquant
  prices <- suppressMessages(
    tidyquant::tq_get(symbol,
                      from = as.character(as.Date(to_date) - 7),
                      to   = as.character(to_date))
  )
  price <- if (nrow(prices) > 0) prices$adjusted[nrow(prices)] else NA_real_

  pe  <- if (!is.na(eps) && eps > 0) round(price / eps, 2) else NA_real_

  pb  <- if (!is.na(shares.out) && shares.out > 0 && !is.na(equity))
    round(price / (equity / shares.out), 2) else NA_real_

  div <- if (!is.na(div.per.share) && !is.na(price) && price > 0)
    round(div.per.share / price * 100, 2) else NA_real_

  c(CIK            = as.numeric(cik),
    EPS            = round(eps,        2),
    NetIncome      = round(net.income, 0),
    Revenue        = round(revenue,    0),
    ROE            = roe,
    ROA            = roa,
    DE             = de,
    CurrentRatio   = current.ratio,
    GrossMargin    = gross.margin,
    OperatingMargin = operating.margin,
    NetMargin      = net.margin,
    PE             = pe,
    PB             = pb,
    DIV            = div)
}

#' Look up a company's SEC Central Index Key from its ticker symbol
#'
#' Translates a stock ticker symbol into the corresponding SEC EDGAR Central
#' Index Key (CIK). The CIK is a unique numerical identifier assigned by the
#' SEC to every company that files with EDGAR. It is required for all
#' subsequent EDGAR API calls.
#'
#' @param symbol A character string containing the stock ticker symbol
#'   (e.g. \code{"AAPL"}). Case-insensitive.
#'
#' @return A character string containing the CIK number (without zero-padding).
#'
#' @details
#' Data is retrieved from \url{https://www.sec.gov/files/company_tickers.json},
#' a publicly maintained mapping file updated by the SEC. No API key is
#' required.
#'
#' The SEC requests that automated tools identify themselves via a User-Agent
#' header. Set your identifier once per session with:
#' \code{options(edgarfundamentals.user_agent = "Your Name your@@email.com")}
#'
#' @examples
#' \dontrun{
#' options(edgarfundamentals.user_agent = "Jane Smith jane@@example.com")
#' get_cik("AAPL")
#' get_cik("LLY")
#' }
#'
#' @importFrom httr GET add_headers content
#' @importFrom jsonlite fromJSON
#' @importFrom dplyr filter pull
#' @export
get_cik <- function(symbol) {
  tickers.raw <- httr::GET(
    "https://www.sec.gov/files/company_tickers.json",
    edgar_ua()
  ) |>
    httr::content(as = "text", encoding = "UTF-8") |>
    jsonlite::fromJSON()

  tickers.df <- do.call(rbind, lapply(tickers.raw, as.data.frame))

  match.row <- tickers.df[toupper(tickers.df$ticker) == toupper(symbol), ]

  if (nrow(match.row) == 0) {
    stop("Ticker '", symbol, "' not found in SEC EDGAR. ",
         "Verify the symbol is a US-listed company.")
  }

  as.character(match.row$cik_str[1])
}

---
title: "Fundamental Analysis with edgarfundamentals"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Fundamental Analysis with edgarfundamentals}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  eval     = FALSE
)
```

## Overview

`edgarfundamentals` provides a simple, ticker-based interface for retrieving
fundamental financial ratios directly from SEC EDGAR 10-K filings. No API key
or paid subscription is required. All data comes from the SEC's official XBRL
API at `data.sec.gov`, which is free, stable, and government-maintained.

The package provides four functions:

- `get_cik()` -- translates a ticker to its SEC Central Index Key
- `get_fundamentals()` -- retrieves ratios for a single stock
- `get_fundamentals_batch()` -- retrieves ratios for a portfolio of stocks
- `get_filing_history()` -- retrieves recent SEC filing history for a stock

## Setup

Before making any API calls, set your User-Agent string. The SEC requests that
automated tools identify themselves so they can contact you if your scripts
cause issues with their servers.

```{r user-agent}
library(edgarfundamentals)

options(edgarfundamentals.user_agent = "Jane Smith jane@example.com")
```

Replace the name and email with your own. You only need to do this once per R
session.

## Looking Up a CIK

Every EDGAR API call requires a CIK number. `get_cik()` handles this
translation automatically inside the other functions, but you can call it
directly if you need the CIK for other purposes.

```{r get-cik}
get_cik("LLY")   # Eli Lilly
get_cik("LMT")   # Lockheed Martin
```

## Fundamentals for a Single Stock

`get_fundamentals()` returns a named vector of key financial ratios derived
from the most recent 10-K filing on or before `to_date`.

```{r single}
get_fundamentals("LLY", to_date = "2024-12-31")
```

The returned vector contains:

| Name | Description |
|------|-------------|
| `CIK` | SEC Central Index Key |
| `EPS` | Diluted Earnings Per Share (USD) |
| `NetIncome` | Net Income (USD) |
| `Revenue` | Total Revenue (USD) |
| `ROE` | Return on Equity (%) |
| `ROA` | Return on Assets (%) |
| `DE` | Debt-to-Equity ratio |
| `CurrentRatio` | Current Assets / Current Liabilities |
| `GrossMargin` | Gross Profit as a percentage of Revenue (%) |
| `OperatingMargin` | Operating Income as a percentage of Revenue (%) |
| `NetMargin` | Net Income as a percentage of Revenue (%) |
| `PE` | Price-to-Earnings ratio |
| `PB` | Price-to-Book ratio |
| `DIV` | Dividend Yield (%) |

Because ratios come from annual 10-K filings, they reflect the most recently
completed fiscal year -- not real-time values. The PE ratio is the only
computed value that uses a live market price (via Yahoo Finance), so it will
change daily even when the underlying 10-K data does not.

## Fundamentals for a Portfolio

`get_fundamentals_batch()` accepts a vector of tickers and returns a tidy data
frame with one row per stock. If retrieval fails for any ticker, that row
contains `NA` values and a message is printed -- the batch continues rather
than stopping.

```{r batch}
healthcare <- c("UNH", "PFE", "MRK", "ABT", "LLY", "CVS", "AMGN")
defense    <- c("LMT", "RTX", "NOC", "GD",  "HII", "LHX", "LDOS")

healthcare.fund <- get_fundamentals_batch(healthcare, to_date = "2024-12-31")
defense.fund    <- get_fundamentals_batch(defense,    to_date = "2024-12-31")

healthcare.fund
defense.fund
```

Expect retrieval to take approximately 20--30 seconds for 14 stocks. A 0.5
second pause is inserted after each API call to respect the SEC rate limit.

## Screening by Fundamental Criteria

Once you have the data frame, standard `dplyr` operations apply directly.

```{r screening}
library(dplyr)

# Growth screen: high EPS stocks relative to peers signal strong earnings
healthcare.fund |> arrange(desc(EPS))

# Value screen: low PE and low PB suggest the market is pricing the stock cheaply
# High dividend yield is also characteristic of value stocks
defense.fund |> filter(PE < 20 & PB < 3) |> select(symbol, PE, PB, DIV, ROE)

# GARP screen (Growth at a Reasonable Price): PE below EPS
# Stocks where the market is not overcharging for growth
healthcare.fund |> filter(EPS > 0 & PE < EPS) |> select(symbol, PE, EPS)

# Profitability screen: strong margins signal pricing power and operational efficiency
bind_rows(healthcare.fund, defense.fund) |>
  filter(GrossMargin > 40 & OperatingMargin > 15) |>
  arrange(desc(NetMargin)) |>
  select(symbol, GrossMargin, OperatingMargin, NetMargin, ROA)

# Liquidity screen: current ratio above 1.5 and low leverage
bind_rows(healthcare.fund, defense.fund) |>
  filter(CurrentRatio > 1.5 & DE < 1) |>
  arrange(desc(CurrentRatio)) |>
  select(symbol, CurrentRatio, DE, ROE)
```

## Checking Filing History

`get_filing_history()` retrieves recent EDGAR filings for a company, which is
useful for verifying data availability or for finding the accession numbers
needed to access the full text of specific reports.

```{r filing-history}
# Five most recent annual reports for Lockheed Martin
get_filing_history("LMT", form_type = "10-K", n = 5)

# Four most recent quarterly reports for Eli Lilly
get_filing_history("LLY", form_type = "10-Q", n = 4)
```

## A Note on Data Limitations

EDGAR XBRL data is only as good as what companies report. A small number of
companies use non-standard XBRL tag names for common concepts. The package
automatically tries fallback tags when the primary tag returns no data, but
in rare cases a ratio may still be `NA`. When this occurs, verifying the
company's most recent 10-K directly on `edgar.sec.gov` will confirm whether
the data exists under a different tag name.

Additionally, EDGAR does not provide market-based ratios directly. Price-to-Book
and Dividend Yield both require a current share price, which comes from Yahoo
Finance via `tidyquant`. This means PE, PB, and DIV all reflect today's price
against the most recent annual filing data, which is standard practice for
trailing ratios.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_filing_history.R
\name{get_filing_history}
\alias{get_filing_history}
\title{Retrieve recent SEC filing history for a company}
\usage{
get_filing_history(symbol, form_type = "10-K", n = 5)
}
\arguments{
\item{symbol}{A character string containing the stock ticker symbol
(e.g. \code{"AAPL"}). Case-insensitive.}

\item{form_type}{A character string specifying the SEC form type to
retrieve. Common values are \code{"10-K"} (annual report),
\code{"10-Q"} (quarterly report), and \code{"8-K"} (current report).
Defaults to \code{"10-K"}.}

\item{n}{An integer specifying the maximum number of filings to return.
Defaults to \code{5}.}
}
\value{
A data frame with one row per filing and the following columns:
\describe{
  \item{symbol}{The ticker symbol passed to the function.}
  \item{accession_number}{The SEC accession number uniquely identifying
    the filing (e.g. \code{"0000320193-24-000123"}).}
  \item{filing_date}{The date the filing was submitted to EDGAR.}
  \item{report_date}{The period-end date covered by the filing.}
  \item{form}{The form type as recorded in EDGAR.}
  \item{primary_document}{The filename of the primary HTML document within
    the filing.}
}
}
\description{
Returns a data frame listing the most recent EDGAR filings of a specified
form type for a given company. Useful for verifying that a company has
filed the expected number of 10-K or 10-Q reports, or for retrieving
accession numbers needed to access specific filings.
}
\details{
Data is retrieved from the SEC EDGAR submissions API
(\url{https://data.sec.gov/submissions/}). The API returns up to the 1,000
most recent filings across all form types; older filings may not appear.

Set your User-Agent once per session:
\code{options(edgarfundamentals.user_agent = "Your Name your@email.com")}
}
\examples{
\dontrun{
options(edgarfundamentals.user_agent = "Jane Smith jane@example.com")

# Five most recent annual reports for Lockheed Martin
get_filing_history("LMT", form_type = "10-K", n = 5)

# Most recent quarterly reports for Eli Lilly
get_filing_history("LLY", form_type = "10-Q", n = 4)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edgarfundamentals-package.R
\docType{package}
\name{edgarfundamentals-package}
\alias{edgarfundamentals}
\alias{edgarfundamentals-package}
\title{edgarfundamentals: Retrieve Fundamental Financial Data from SEC 'EDGAR'}
\description{
Provides a simple, ticker-based interface for retrieving fundamental financial data from the United States Securities and Exchange Commission's 'EDGAR' 'XBRL' API \url{https://www.sec.gov/edgar/sec-api-documentation}. Functions return key financial ratios including earnings per share, return on equity, return on assets, debt-to-equity, current ratio, gross margin, operating margin, net margin, price-to-earnings, price-to-book, and dividend yield for any publicly traded U.S. company. Data is sourced directly from company 10-K annual filings, requiring no API key or paid subscription. Designed for use in quantitative finance courses and research workflows.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/robschumaker/edgarfundamentals}
  \item Report bugs at \url{https://github.com/robschumaker/edgarfundamentals/issues}
}

}
\author{
\strong{Maintainer}: Robert Schumaker \email{rob.schumaker@gmail.com} (\href{https://orcid.org/0000-0003-2977-1856}{ORCID})

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_cik.R
\name{get_cik}
\alias{get_cik}
\title{Look up a company's SEC Central Index Key from its ticker symbol}
\usage{
get_cik(symbol)
}
\arguments{
\item{symbol}{A character string containing the stock ticker symbol
(e.g. \code{"AAPL"}). Case-insensitive.}
}
\value{
A character string containing the CIK number (without zero-padding).
}
\description{
Translates a stock ticker symbol into the corresponding SEC EDGAR Central
Index Key (CIK). The CIK is a unique numerical identifier assigned by the
SEC to every company that files with EDGAR. It is required for all
subsequent EDGAR API calls.
}
\details{
Data is retrieved from \url{https://www.sec.gov/files/company_tickers.json},
a publicly maintained mapping file updated by the SEC. No API key is
required.

The SEC requests that automated tools identify themselves via a User-Agent
header. Set your identifier once per session with:
\code{options(edgarfundamentals.user_agent = "Your Name your@email.com")}
}
\examples{
\dontrun{
options(edgarfundamentals.user_agent = "Jane Smith jane@example.com")
get_cik("AAPL")
get_cik("LLY")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_fundamentals.R
\name{get_fundamentals}
\alias{get_fundamentals}
\title{Retrieve key fundamental financial ratios for a single stock}
\usage{
get_fundamentals(symbol, to_date = as.character(Sys.Date()))
}
\arguments{
\item{symbol}{A character string containing the stock ticker symbol
(e.g. \code{"LLY"}). Case-insensitive.}

\item{to_date}{A character string in \code{"YYYY-MM-DD"} format. The
function returns ratios from the most recent 10-K with a period end date
on or before this date. Defaults to today's date.}
}
\value{
A named numeric vector with the following elements:
\describe{
  \item{CIK}{SEC Central Index Key -- the company's unique EDGAR identifier.}
  \item{EPS}{Diluted Earnings Per Share (USD per share) from the most recent
    qualifying 10-K. Falls back to basic EPS if diluted is unavailable.}
  \item{NetIncome}{Net income attributable to the company (USD).}
  \item{Revenue}{Total revenue (USD).}
  \item{ROE}{Return on Equity as a percentage (NetIncome / StockholdersEquity
    * 100). A measure of how efficiently the company generates profit from
    shareholder capital.}
  \item{ROA}{Return on Assets as a percentage (NetIncome / TotalAssets
    * 100). A measure of how efficiently the company uses its assets.}
  \item{DE}{Debt-to-Equity ratio (LongTermDebt / StockholdersEquity). A
    measure of financial leverage.}
  \item{CurrentRatio}{Current Assets divided by Current Liabilities. A
    measure of short-term liquidity; values above 1 indicate the company
    can cover near-term obligations.}
  \item{GrossMargin}{Gross Profit as a percentage of Revenue. Measures
    pricing power and production efficiency.}
  \item{OperatingMargin}{Operating Income as a percentage of Revenue.
    Measures operational efficiency before interest and taxes.}
  \item{NetMargin}{Net Income as a percentage of Revenue. Bottom-line
    profitability after all expenses.}
  \item{PE}{Price-to-Earnings ratio, computed by dividing the most recent
    adjusted closing price (from Yahoo Finance via tidyquant) by EPS.
    Returns \code{NA} if EPS is zero or negative.}
  \item{PB}{Price-to-Book ratio, computed by dividing the most recent
    adjusted closing price by Book Value per Share (StockholdersEquity /
    SharesOutstanding). Returns \code{NA} if shares outstanding is zero
    or unavailable.}
  \item{DIV}{Dividend Yield as a percentage (DividendsPerShareDeclared /
    Price * 100). Returns \code{NA} if no dividend data is available.}
}
}
\description{
Pulls fundamental financial data directly from a company's most recent
annual 10-K filing in SEC EDGAR and computes key financial ratios. No API
key or paid subscription is required.
}
\details{
Financial statement values are extracted from XBRL-tagged 10-K filings via
the SEC EDGAR companyfacts API
(\url{https://data.sec.gov/api/xbrl/companyfacts/}). Because data comes
from annual filings, ratios reflect the most recently completed fiscal year
ending on or before \code{to_date}, not real-time values.

Fallback XBRL tags are attempted automatically when a company uses a
non-standard tag name for a concept. A courtesy pause of 0.5 seconds is
inserted after the companyfacts API call to respect the SEC's rate limit
of 10 requests per second.

Set your User-Agent once per session:
\code{options(edgarfundamentals.user_agent = "Your Name your@email.com")}
}
\examples{
\dontrun{
options(edgarfundamentals.user_agent = "Jane Smith jane@example.com")

# Fundamentals for Eli Lilly as of end of 2024
get_fundamentals("LLY", "2024-12-31")

# Fundamentals as of today
get_fundamentals("JNJ")
}

}
